from fastsdk.definitions.ai_model import AIModelDescription
from fastsdk.definitions.enums import ModelDomainTag
from fastsdk.web.service_client import ServiceClient
from pydantic import BaseModel, Field

from media_toolkit import ImageFile
from socaity.settings import DEFAULT_SOCAITY_URL

# Service Client for SAM 2
srvc_sam2 = ServiceClient(
    service_urls={
        "socaity": f"{DEFAULT_SOCAITY_URL}/sam2",
        "replicate": {"version": "fe97b453a6455861e3bac769b441ca1f1086110da7466dbb65cf1eecfd60dc83" },
    },
    service_name="sam2",
    model_description=AIModelDescription(
        model_name="SAM 2",
        model_domain_tags=[ModelDomainTag.IMAGE, ModelDomainTag.MISC],
    ),
    upload_to_cloud_threshold_mb=0
)

# Input schema for the SAM 2 endpoint
class SAM2Input(BaseModel):
    points_per_side: int = Field(default=32, description="Number of points per side for segmentation.")
    pred_iou_thresh: float = Field(default=0.88, description="Prediction IoU threshold.")
    stability_score_thresh: float = Field(default=0.95, description="Stability score threshold.")
    use_m2m: bool = Field(default=True, description="Enable model-to-model interaction for segmentation.")


# Add endpoint to the service client
srvc_sam2.add_endpoint(
    endpoint_route="/segment",
    body_params=SAM2Input(),
    file_params={"image": ImageFile},
    refresh_interval_s=1,
)



from fastsdk.definitions.ai_model import AIModelDescription
from fastsdk.definitions.enums import ModelDomainTag
from fastsdk.web.service_client import ServiceClient
from pydantic import BaseModel, Field

from media_toolkit import ImageFile
from socaity.settings import DEFAULT_SOCAITY_URL

# Service Client for SAM 2
srvc_photomaker = ServiceClient(
    service_urls={
        "socaity": f"{DEFAULT_SOCAITY_URL}/tencentarc-photomaker",
        "replicate": {"version": "467d062309da518648ba89d226490e02b8ed09b5abc15026e54e31c5a8cd0769" },
    },
    service_name="tencentarc-photomaker",
    model_description=AIModelDescription(
        model_name="tencentarc-photomaker",
        model_domain_tags=[ModelDomainTag.IMAGE],
    ),
    upload_to_cloud_threshold_mb=0
)

# Input schema for the SAM 2 endpoint
class PhotoMakerInput(BaseModel):
    prompt: str = Field(default="", description="What kind of image do you want to create?")
    style_name: str = Field(default="", description="Use a specific style template")
    negative_prompt: str = Field(default="nsfw, lowres, bad anatomy, bad hands, text, error, missing fingers, extra digit, fewer digits, cropped, worst quality, low quality, normal quality, jpeg artifacts, signature, watermark, username, blurry",
                                 description="Negative Prompt. The negative prompt should NOT contain the trigger word.")
    num_steps: int = Field(default=50, gt=0, lt=100)
    style_strength_ratio: float = Field(default=35, gt=0, le=50)
    num_outputs: int = Field(default=1, gt=0, le=4)
    guidance_scale: float = Field(default=5, ge=0, le=10)
    seed: int = Field(default=32, ge=0)
    disable_safety_checker: bool = Field(default=False, description="Disable safety checker")


# Add endpoint to the service client
srvc_photomaker.add_endpoint(
    endpoint_route="/generate",
    query_params=PhotoMakerInput(),
    file_params={
        "input_image": ImageFile,
        "input_image2": ImageFile,
        "input_image3": ImageFile,
        "input_image4": ImageFile,
    },
    refresh_interval_s=1,
)



import random
from dataclasses import dataclass
from enum import Enum
from typing import Union, List

from fastsdk.jobs.threaded.internal_job import InternalJob
from fastsdk.fast_sdk import fastSDK, fastJob
from fastsdk.utils import get_function_parameters_as_dict
from media_toolkit import ImageFile
from .tencentarc_photomaker_service_client import srvc_photomaker


class TencentPhotoMakerStyleTemplate(Enum):
    CINEMATIC = "Cinematic"
    DISNEY = "Disney Charactor"
    DIGITAL_ART = "Digital Art"
    PHOTOGRAPHIC = "Photographic"
    FANTASY_ART = "Fantasy art"
    NEONPUNK = "Neonpunk"
    ENHANCE = "Enhance"
    COMIC = "Comic book"
    LOWPOLY = "Lowpoly"
    LINEART = "Lineart"

@fastSDK(service_client=srvc_photomaker)
class TencentPhotoMaker:
    """
    Base implementation for SAM 2, the Segment Anything v2 model from Meta.
    """
    def __init__(self):
        self._socaity_avatar_reference_image_1 = "https://socaityfiles.blob.core.windows.net/backend-model-meta/socaity_icon_no_shoulders_eyes.ico"

    @fastJob
    def _generate(self, job,
        input_image: Union[str, bytes, ImageFile],
        input_image1: Union[str, bytes, ImageFile],
        input_image2: Union[str, bytes, ImageFile],
        input_image3: Union[str, bytes, ImageFile],
        input_image4: Union[str, bytes, ImageFile],
        prompt: str = "",
        style_name: str = None,
        negative_prompt: str = "",
        num_steps: int = 50,
        style_strength_ratio: float = 35,
        num_outputs: int = 1,
        guidance_scale: float = 5,
        seed: int = None,
        disable_safety_checker: bool = False,
        **kwargs) -> Union[ImageFile, List[ImageFile]]:
        """
        Create photos, paintings and avatars for anyone in any style within seconds.
        :param image: URL or path to the input image.
        :param points_per_side: Number of points per side for segmentation.
        :param pred_iou_thresh: Prediction IoU threshold.
        :param stability_score_thresh: Stability score threshold.
        :param use_m2m: Whether to enable model-to-model interaction.
        :return: Segmentation results as a dictionary.
        """
        if not seed or not isinstance(seed, int) or seed < 0:
            seed = random.randint(0, 1000)

        response = job.request(
            endpoint_route="/generate",
            input_image=input_image,
            input_image1=input_image1,
            input_image2=input_image2,
            input_image3=input_image3,
            input_image4=input_image4,
            prompt=prompt,
            style_name=style_name,
            negative_prompt=negative_prompt,
            num_steps=num_steps,
            style_strength_ratio=style_strength_ratio,
            num_outputs=num_outputs,
            guidance_scale=guidance_scale,
            seed=seed,
            disable_safety_checker=disable_safety_checker,
            **kwargs,
        )
        res = response.get_result()
        if response.error:
            raise Exception(f"Error in photomaker: {response.error}")

        if isinstance(res, list):
            if len(res) == 1:
                return ImageFile().from_any(res[0])
            return [ImageFile().from_any(img) for img in res]

        return ImageFile().from_any(res)

    def generate(self,
        input_image: Union[str, bytes, ImageFile],
        input_image1: Union[str, bytes, ImageFile] = None,
        input_image2: Union[str, bytes, ImageFile] = None,
        input_image3: Union[str, bytes, ImageFile] = None,
        input_image4: Union[str, bytes, ImageFile] = None,
        prompt: str = "",
        style_name: str = TencentPhotoMakerStyleTemplate.PHOTOGRAPHIC,
        negative_prompt: str = "nsfw, lowres, bad anatomy, bad hands, text, error, missing fingers, extra digit, fewer digits, cropped, worst quality, low quality, normal quality, jpeg artifacts, signature, watermark, username, blurry",
        num_steps: int = 50,
        style_strength_ratio: float = 35,
        num_outputs: int = 1,
        guidance_scale: float = 5,
        seed: int = None,
        disable_safety_checker: bool = False,
        *args,
        **kwargs) -> InternalJob:
        """
        Perform image segmentation using SAM 2.
        :param image: URL or path to the input image.
        """
        # Check if style_name is an instance of TencentPhotoMakerStyleTemplate or can be converted to
        if isinstance(style_name, str):
            try:
                style_name = TencentPhotoMakerStyleTemplate(style_name)
            except ValueError:
                print(f"Invalid style name: {style_name} using photographic style")
                style_name = TencentPhotoMakerStyleTemplate.PHOTOGRAPHIC

        # convert style name to string
        if isinstance(style_name, TencentPhotoMakerStyleTemplate):
            style_name = style_name.value

        pams = get_function_parameters_as_dict(
            self._generate,
            exclude_param_names="job",
            func_kwargs=locals()
        )
        return self._generate(**pams)

    def create_socaity_avatar(
            self,
            input_image: Union[str, bytes, ImageFile],
            num_outputs: int = 1
    ):
        """
        Create a socaity avatar from an image.
        :param input_image: Your profile picture that gets converted. Best results with a close up portrait.
        """
        return self.generate(
            input_image=input_image,
            input_image1=self._socaity_avatar_reference_image_1,
            prompt="""
            A minimalist, sci-fi img featuring a close-up of Rick's face from Rick and Morty.
            The eyes are glowing neon green bioluminescent. The background is entirely black.
            The face is showcasing a vibrant neon-green lime palette, rendered in an anime-style illustration with 4k detail.
            8bit style. Icon.
            """,
            style_name=TencentPhotoMakerStyleTemplate.NEONPUNK,
            num_outputs=num_outputs
        )



import random

from fastsdk.jobs.threaded.internal_job import InternalJob
from fastsdk.fast_sdk import fastSDK, fastJob
from socaity.api.text.chat.i_chat import _BaseChat
from socaity.api.text.chat.llama3.meta_llama_3_code_service_client import srvc_codellama_13b, srvc_codellama_70b_python


class _BaseMetaLlama3_Code(_BaseChat):
    """
    Base version of Llama 3, an 8 billion parameter language model from Meta.
    """
    @fastJob
    def _chat(self, job,
              prompt: str,
              system_prompt: str,
              max_new_tokens: int = 512,
              temperature: float = 0.5,
              top_p: float =0.9,
              length_penalty: float = 1.15,
              stop_sequences: str = "<|end_of_text|>,<|eot_id|>",
              presence_penalty: float = 0.0,
              frequency_penalty: float = 0.2,
              repeat_penalty: float = 1.1,
              seed: int = None,
              **kwargs) -> str:

        if seed is None or not isinstance(seed, int):
            seed = random.randint(0, 1000000)

        response = job.request(
            endpoint_route="/chat",
            prompt=prompt,
            system_prompt=system_prompt,
            max_new_tokens=max_new_tokens,
            temperature=temperature,
            length_penalty=length_penalty,
            top_p=top_p,
            stop_sequences=stop_sequences,
            presence_penalty=presence_penalty,
            frequency_penalty=frequency_penalty,
            repeat_penalty=repeat_penalty,
            seed=seed,
            **kwargs,
        )
        result = response.get_result()
        if response.error:
            raise Exception(f"Error in generate_text: {response.error}")

        if isinstance(result, list):
            result = "".join(result)

        return result

    def chat(self, prompt: str, max_tokens: int = 512, temperature: float = 0.5, **kwargs) -> InternalJob:
        """
        Generate text from the provided prompt.
        :param prompt: The input text prompt.
        :param max_tokens: Maximum number of tokens to generate.
        :param temperature: Sampling temperature for generation.
        """
        return self._chat(prompt=prompt, max_tokens=max_tokens, temperature=temperature, **kwargs)


@fastSDK(service_client=srvc_codellama_13b)
class MetaLLama3_13b_code(_BaseMetaLlama3_Code):
    """
    Llama 3, an 8 billion parameter language model from Meta.
    """
    pass

@fastSDK(service_client=srvc_codellama_70b_python)
class MetaLLama3_70b_code_python(_BaseMetaLlama3_Code):
    """
    Llama 3, a language model from Meta.
    """
    pass


from abc import abstractmethod
from typing import Union, List

import media_toolkit as mt
import numpy as np

class _BaseImage2Text:

    @abstractmethod
    def image2text(self, image: Union[np.array, bytes, str, mt.ImageFile], *args, **kwargs) -> str:
        """
        Creates an image caption or description from an image.
        :param text: The text to convert to an image
        :return: The image
        """
        raise NotImplementedError("Please implement this method")

    # alias
    caption_image = image2text

    def visual_question_answering(self, image: Union[np.array, bytes, str, mt.ImageFile], question: str) -> str:
        """
        Answers a question about an image.
        :param image: The image to ask a question about
        :param question: The question to ask
        :return: The answer to the question
        """
        raise NotImplementedError("Please implement this method")

    def image_text_matching(self, image: Union[np.array, bytes, str, mt.ImageFile], caption: str) -> str:
        """
        Measures how well an image description is matched by the input image.
        :param image: The image to match
        :param caption: The caption to match
        :return: The matching score
        """
        raise NotImplementedError("Please implement this method")



def image2text(image: Union[np.array, bytes, str, mt.ImageFile], model="blip", service="socaity", *args, **kwargs) -> str:
    from .blib.blib import Blip
    names_cl = {
        "blib": Blip,
    }
    names_cl = {k.lower().replace("-", ""): v for k, v in names_cl.items()}
    mdlcl = names_cl.get(model.lower().replace("-", ""), Blip)
    return mdlcl(service=service).image2text(image=image, *args, **kwargs)

def image_captioning(image: Union[np.array, bytes, str, mt.ImageFile], model="blip", service="socaity", *args, **kwarg) -> str:
    return image2text(image=image, model=model, service=service, *args, **kwarg)

def visual_question_answering(image: Union[np.array, bytes, str, mt.ImageFile], question: str, model="blip", service="socaity", *args, **kwargs) -> str:
    from .blib.blib import Blip
    names_cl = {
        "blib": Blip,
    }
    names_cl = {k.lower().replace("-", ""): v for k, v in names_cl.items()}
    mdlcl = names_cl.get(model.lower().replace("-", ""), Blip)
    return mdlcl(service=service).visual_question_answering(image=image, question=question, *args, **kwargs)

def image_text_matching(image: Union[np.array, bytes, str, mt.ImageFile], caption: str, model="blip", service="socaity", *args, **kwargs) -> str:
    from .blib.blib import Blip
    names_cl = {
        "blib": Blip,
    }
    names_cl = {k.lower().replace("-", ""): v for k, v in names_cl.items()}
    mdlcl = names_cl.get(model.lower().replace("-", ""), Blip)
    return mdlcl(service=service).image_text_matching(image=image, caption=caption, *args, **kwargs)


from .i_chat import chat
from .llama3.meta_llama_3 import MetaLLama3_8b, MetaLLama3_70b
from .llama3.meta_llama_3_code import MetaLLama3_13b_code, MetaLLama3_70b_code_python
from .llama3.meta_llama_3_instruct import MetaLLama3_70b_instruct, MetaLLama3_8b_instruct
from .deepseek import DeepSeekR1



from pydantic import BaseModel, Field

class DeepSeekR1_Input(BaseModel):
    prompt: str = Field(default="")
    max_tokens: int = Field(default=20480, gt=0)
    temperature: float = Field(default=0.1, ge=0, le=1)
    presence_penalty: float = Field(default=0.0)
    frequency_penalty: float = Field(default=0.0)
    top_p: float = Field(default=1.0, ge=0, le=1)




import random

from fastsdk.jobs.threaded.internal_job import InternalJob
from fastsdk.fast_sdk import fastSDK, fastJob
from socaity.api.text.chat.i_chat import _BaseChat
from socaity.api.text.chat.llama3.meta_llama_3_service_client import srvc_meta_llama_3_8b, srvc_meta_llama_3_70b


class _BaseMetaLlama3(_BaseChat):
    """
    Base version of Llama 3, an 8 billion parameter language model from Meta.
    """
    @fastJob
    def _chat(self, job,
              prompt: str, max_tokens: int = 512, temperature: float = 0.5, top_p: float =0.9,
              presence_penalty: float = 1.15, frequency_penalty: float = 0.2,
              prompt_template: str = "{prompt}",
              seed: int = None,
              **kwargs) -> str:

        if seed is None or not isinstance(seed, int):
            seed = random.randint(0, 1000000)

        response = job.request(
            endpoint_route="/chat",
            prompt=prompt,
            max_tokens=max_tokens,
            temperature=temperature,
            top_p=top_p,
            presence_penalty=presence_penalty,
            frequency_penalty=frequency_penalty,
            seed=seed,
            prompt_template=prompt_template,
            **kwargs,
        )
        result = response.get_result()
        if response.error:
            raise Exception(f"Error in generate_text: {response.error}")

        if isinstance(result, list):
            result = "".join(result)

        return result

    def chat(self, prompt: str, max_tokens: int = 512, temperature: float = 0.5, **kwargs) -> InternalJob:
        """
        Generate text from the provided prompt.
        :param prompt: The input text prompt.
        :param max_tokens: Maximum number of tokens to generate.
        :param temperature: Sampling temperature for generation.
        """
        return self._chat(prompt=prompt, max_tokens=max_tokens, temperature=temperature, **kwargs)


@fastSDK(service_client=srvc_meta_llama_3_8b)
class MetaLLama3_8b(_BaseMetaLlama3):
    """
    Llama 3, an 8 billion parameter language model from Meta.
    """
    pass

@fastSDK(service_client=srvc_meta_llama_3_70b)
class MetaLLama3_70b(_BaseMetaLlama3):
    """
    Llama 3, an 8 billion parameter language model from Meta.
    """
    pass


import os

from socaity import DeepSeekR1

chat_model = DeepSeekR1()
#chat_model = DeepSeekR1(service="replicate", api_key=os.getenv("REPLICATE_API_KEY", None))
# fluxs = FluxSchnell(service="socaity_local", api_key=os.getenv("SOCAITY_API_KEY", None))

def test_chat():
    prompt = "Write a poem with 3 sentences why an SDK is so much better than plain web requests."
    fj = chat_model.chat(
        prompt=prompt
    )
    generated_text = fj.get_result()
    print(generated_text)


if __name__ == "__main__":
    test_chat()


from socaity.api.audio import SpeechCraft, text2voice
from socaity.api.image.text2img import FluxSchnell, text2img
from socaity.api.image.img2img import Face2Face, image_segmentation, Sam2, TencentPhotoMaker, TencentPhotoMakerStyleTemplate
from socaity.api.video.text2video import HunyuanVideo
from socaity.api.text import *



from .face2face.face2face_api import Face2Face
from .face2face.face2face_service_client import srvc_face2face
from .image_segmentation import Sam2, image_segmentation
from .style_transfer import TencentPhotoMaker, TencentPhotoMakerStyleTemplate


from fastsdk.definitions.ai_model import AIModelDescription
from fastsdk.definitions.enums import ModelDomainTag
from fastsdk.web.service_client import ServiceClient
from socaity.settings import DEFAULT_SOCAITY_URL, DEFAULT_REPLICATE_URL

srvc_flux_schnell = ServiceClient(
    service_urls={
        "socaity": f"{DEFAULT_SOCAITY_URL}/flux-schnell",
        "socaity_local": "http://localhost:8000/api/v0/flux-schnell",
        "replicate": f"{DEFAULT_REPLICATE_URL}/black-forest-labs/flux-schnell/predictions",
    },
    service_name="flux-schnell",
    model_description=AIModelDescription(
        model_name="flux-schnell",
        model_domain_tags=[ModelDomainTag.IMAGE, ModelDomainTag.TEXT]
    )
)

# Endpoint definitions

from pydantic import BaseModel, Field


class FluxText2ImgPostParams(BaseModel):
    prompt: str = Field(default="")
    aspect_ratio: str = Field(default="1:1")
    num_outputs: int = Field(default=1)
    num_inference_steps: int = Field(default=4, gt=0, lt=5)
    seed: int = Field(default=None)
    output_format: str = Field(default="jpg")
    disable_safety_checker: bool = Field(default=False)
    go_fast: bool = Field(default=False)


srvc_flux_schnell.add_endpoint(
    endpoint_route="/text2img",
    query_params=FluxText2ImgPostParams(),
    refresh_interval_s=2
)


from fastsdk.utils import get_function_parameters_as_dict


def get_model_instance(
        name_instance_dict: dict,
        model_name: str,
        service: str = "socaity",
        default_instance=None):
    # get model class from name_instance_dict
    names_cl = {k.lower().replace("-", ""): v for k, v in name_instance_dict.items()}
    if not default_instance:
        default_instance = list(name_instance_dict.values())[0]

    # instantiate class
    mdlcl = names_cl.get(model_name.lower().replace("-", ""), default_instance)
    return mdlcl(service=service)


def execute_job_function(job_func: callable, pam_locals: dict):
    pams = get_function_parameters_as_dict(job_func,exclude_param_names="job", func_kwargs=pam_locals)
    return job_func(**pams)



import os

from socaity import FluxSchnell

#fluxs = FluxSchnell(service="replicate", api_key=os.getenv("REPLICATE_API_KEY", None))
# fluxs = FluxSchnell(service="socaity_local", api_key=os.getenv("SOCAITY_API_KEY", None))
fluxs = FluxSchnell(service="socaity", api_key=os.getenv("SOCAITY_API_KEY", None))

def test_text2img():
    prompt = (
        """
        A product photo of a studio microphone of which the head (grill) is the head of Rick (from Rick and Morty). 
        The microphone is usb connected to a tablet. On the tablet in big letters is written "WHISPER".
        The scene is in a cozy laidback audio studio lit with deep purple and lime colors. The background is blurry. The spotlight is on the microphone.
        The artwork is minimalistic yet striking and cinematic, showcasing a vibrant neon-green lime palette, rendered in an anime-style illustration with 4k detail. 
        Influenced by the artistic styles of Simon Kenny, Giorgetto Giugiaro, Brian Stelfreeze, and Laura Iverson
        """
    )
    fj = fluxs.text2img(
        text=prompt, aspect_ratio="1:1", num_outputs=1, num_inference_steps=4, output_format="png",
        disable_safety_checker=True, go_fast=False
    )
    imgs = fj.get_result()
    if not isinstance(imgs, list):
        imgs = [imgs]

    for i, img in enumerate(imgs):
        img.save(f"test_files/output/text2img/test_fluxs_text2img_{i}.png")

if __name__ == "__main__":
    test_text2img()


from fastsdk.jobs.threaded.internal_job import InternalJob
from fastsdk.fast_sdk import fastSDK, fastJob
from socaity.api.text.chat.i_chat import _BaseChat
from .deepseek_r1_service_client import srvc_deep_seek_r1

class _BaseDeepSeekR1(_BaseChat):
    """
    Base version of Llama 3, an 8 billion parameter language model from Meta.
    """
    @fastJob
    def _chat(self, job,
              prompt: str,
              max_tokens: int = 20480,
              temperature: float = 0.1,
              presence_penalty: float = 0.0,
              frequency_penalty: float = 0.0,
              top_p: float = 1.0,
              **kwargs
        ) -> str:
        response = job.request(
            endpoint_route="/chat",
            prompt=prompt,
            max_tokens=max_tokens,
            temperature=temperature,
            presence_penalty=presence_penalty,
            frequency_penalty=frequency_penalty,
            top_p=top_p,
            **kwargs,
        )
        result = response.get_result()
        if response.error:
            raise Exception(f"Error in generate_text: {response.error}")

        if isinstance(result, list):
            result = "".join(result)

        return result

    def chat(self,
        prompt: str,
        max_tokens: int = 20480,
        temperature: float = 0.1,
        presence_penalty: float = 0.0,
        frequency_penalty: float = 0.0,
        top_p: float = 1.0, **kwargs) -> InternalJob:
        """
        Generate text from the provided prompt.
        :param prompt: The input text prompt.
        :param max_tokens: Maximum number of tokens to generate.
        :param temperature: Sampling temperature for generation.
        """
        return self._chat(
            prompt=prompt, max_tokens=max_tokens, temperature=temperature,
            presence_penalty=presence_penalty, frequency_penalty=frequency_penalty, top_p=top_p,
            **kwargs
        )

    def pretty(self, answer: str, exclude_thoughts: bool = True):
        if exclude_thoughts:
            pos_think = answer.find("<think>")
            end_think = answer.rfind("</think>")
            if pos_think > -1:
                pos_think += len("<think>")
                answer = answer[:pos_think] + answer[end_think:]
                #thoughts = answer[pos_think+len("<think>"):end_think]

        return answer


@fastSDK(service_client=srvc_deep_seek_r1)
class DeepSeekR1(_BaseDeepSeekR1):
    """
    Llama 3, an 8 billion parameter language model from Meta.
    """



from abc import abstractmethod
from typing import Union, List

import media_toolkit as mt
from fastsdk.jobs.threaded.internal_job import InternalJob
from socaity.api.utils import get_model_instance


class _BaseText2Image:

    @abstractmethod
    def text2img(self, text, *args, **kwargs) -> Union[mt.ImageFile, List[mt.ImageFile], None]:
        """
        Converts text to an image
        :param text: The text to convert to an image
        :return: The image
        """
        raise NotImplementedError("Please implement this method")


# Factory method for generalized model_hosting_info calling
def text2img(
        text: str, model: str = "flux-schnell", service: str = "socaity", wait_for_result: bool = False, *args, **kwargs
) -> Union[InternalJob, mt.ImageFile, List[mt.ImageFile], None]:
    """
    Creates a beautiful image from the given text.
    :param text: The text to convert to an image.
    :param model: The model to use for the conversion.
    :param service: The service to use for the conversion.
    :param wait_for_result: Whether to wait for the result. If False (default), returns the job.
    """

    from .flux_schnell import FluxSchnell
    names_cl = {"flux-schnell": FluxSchnell}
    mdl = get_model_instance(names_cl, model_name=model, service=service, default_instance=FluxSchnell)
    job = mdl.text2img(text=text, *args, **kwargs)
    if wait_for_result:
        return job.get_result()
    return job



from abc import abstractmethod
from typing import Union, List

import media_toolkit as mt
import numpy as np

class _BaseAudio2Text:
    @abstractmethod
    def transcribe(self, audio: Union[np.array, bytes, str, mt.AudioFile], *args, **kwargs) -> str:
        """
        Creates an image caption or description from an image.
        :param text: The text to convert to an image
        :return: The image
        """
        raise NotImplementedError("Please implement this method")

    # alias
    audio2text = transcribe




def transcribe(audio: Union[np.array, bytes, str, mt.AudioFile],
               model="insanely-fast-whisper",
               service="socaity", *args, **kwargs) -> str:
    from .whisper import InsanelyFastWhisper
    names_cl = {
        "insanely-fast-whisper": InsanelyFastWhisper,
    }
    names_cl = {k.lower().replace("-", ""): v for k, v in names_cl.items()}
    mdlcl = names_cl.get(model.lower().replace("-", ""), InsanelyFastWhisper)
    return mdlcl(service=service).image2text(audio=audio, *args, **kwargs)




import os

from socaity import Sam2

genai = Sam2(service="replicate", api_key=os.getenv("REPLICATE_API_KEY", None))
# fluxs = FluxSchnell(service="socaity_local", api_key=os.getenv("SOCAITY_API_KEY", None))

def test_sam2():
    image = "test_files/face2face/test_face_1.jpg"
    fj = genai.segment(image=image)
    masks = fj.get_result()
    print(masks)


if __name__ == "__main__":
    test_sam2()


from .chat import (chat, MetaLLama3_70b_instruct, MetaLLama3_70b_code_python,
                   MetaLLama3_13b_code, MetaLLama3_70b, MetaLLama3_8b, MetaLLama3_8b_instruct)
from .chat import DeepSeekR1
from .img2text import image2text, visual_question_answering, image_text_matching, image_captioning, Blip
from .audio2text import InsanelyFastWhisper, transcribe



from socaity.api.video import HunyuanVideo

genai = HunyuanVideo()
# genai = HunyuanVideo(service="replicate", api_key=os.getenv("REPLICATE_API_KEY", None))
# genai = HunyuanVideo(service="socaity_local", api_key=os.getenv("SOCAITY_API_KEY", None))


def test_text2video():
    prompt = (
        "Rick (of Rick and Morty) sells a GPU over the counter in a computer store."
        "Rick is standing behind the counter. He already has the GPU in his hand and is not moving within the store."
        "The counter is a designer piece and has the shape of an cloud. "
        "The beginning of the scene is a medium-shot having Rick and the counter in focus."
        "Rick hands over an NVIDIA GPU into the camera."
        "While doing so, the camera quickly zooms-in to a close-up shot of the GPU."
        "Text on the GPU is SOCAITY. "
        "The store if full of shelf with GPUs, RAMs and servers."
        "The store is dark-lit in neon deep-purple light. "
        "One ambient spot-light is over the desk. "
        "The furniture design is in vibrant neon-green lime colors."
        "Sci-fi. Cyberpunk neon-punk. 4k. Cinematic. "
        "Filmed by Robert Zemeckis, Baz Luhrmann, Gerry Anderson"
    )
    fj = genai.text2video(text=prompt, width=512, height=512, video_length=97)
    vid = fj.get_result()
    vid = [vid] if not isinstance(vid, list) else vid
    for i, v in enumerate(vid):
        v.save(f"test_files/output/text2video/test_hunyuan_video_{i+1}.mp4")

if __name__ == "__main__":

    #from media_toolkit import VideoFile
    #v = VideoFile().from_any('https://replicate.delivery/xezq/wUs83jTFvo4xOdTAUarf6VFRtYwF0i3BqyS5drN87lYElFBKA/video.mp4')
    #a = 1

    test_text2video()


from pydantic import BaseModel, Field


class _MetaLlama3_common(BaseModel):
    prompt: str = Field(default="")
    temperature: float = Field(default=0.6, ge=0, le=1)
    top_p: float = Field(default=0.9, ge=0, le=1)
    frequency_penalty: float = Field(default=0.2)
    presence_penalty: float = Field(default=1.15)

class MetaLlama3_Input(_MetaLlama3_common):
    max_tokens: int = Field(default=512, gt=0)
    length_penalty: float = Field(default=1.0, ge=0, le=5)
    prompt_template: str = Field(default="{prompt}")
    seed: int = Field(default=0, ge=0)

class MetaCodeLlama3_Input(_MetaLlama3_common):
    repeat_penalty: float = Field(default=1.0, ge=0, le=2)

class MetaLlama3_InstructInput(BaseModel):
    prompt: str = Field(default="")
    system_prompt: str = Field(default="You are a helpful assistant")
    max_new_tokens: int = Field(default=512, gt=0)
    temperature: float = Field(default=0.6, ge=0, le=1)
    top_p: float = Field(default=0.9, ge=0, le=1)
    length_penalty: float = Field(default=1.0)
    stop_sequences: str = Field(default="<|end_of_text|>,<|eot_id|>")
    presence_penalty: float = Field(default=0.0)
    frequency_penalty: float = Field(default=0.2)
    prompt_template: str = Field(
        default="""<|begin_of_text|><|start_header_id|>system<|end_header_id|>
        You are a helpful assistant<|eot_id|><|start_header_id|>user<|end_header_id|>
        {prompt}<|eot_id|><|start_header_id|>assistant<|end_header_id|>
        """
    )



from fastsdk import ImageFile, VideoFile
from fastsdk.definitions.ai_model import AIModelDescription
from fastsdk.definitions.enums import ModelDomainTag
from fastsdk.web.definitions.service_adress import SocaityServiceAddress
from fastsdk.web.service_client import ServiceClient
from socaity.settings import DEFAULT_SOCAITY_URL


srvc_face2face = ServiceClient(
    service_name="face2face",
    service_description="Instantly swap faces in images and videos. Face restoration and recognition.",
    service_urls={
        "localhost": "localhost:8020/api",
        "runpod": "https://api.runpod.ai/v2/v1n4b07cwp2mbo",
        "runpod_localhost": "http://localhost:8020/",
        "socaity": f"{DEFAULT_SOCAITY_URL}/face2face",
        "socaity_local1": SocaityServiceAddress("http://localhost:8000/api/v0/face2face"),
        "socaity_local": "http://localhost:8000/api/v0/face2face"
    },
    model_description=AIModelDescription(
        model_name="face2face",
        model_domain_tags=[ModelDomainTag.IMAGE, ModelDomainTag.AUDIO]
    )
)
srvc_face2face.add_endpoint(
    endpoint_route="/swap_img_to_img",
    query_params={"enhance_face_model": str},
    file_params={"source_img": ImageFile, "target_img": ImageFile}
)
srvc_face2face.add_endpoint(
    endpoint_route="/swap",
    query_params={"faces": str, "enhance_face_model": str},
    file_params={"media": ImageFile}
)
srvc_face2face.add_endpoint(
    endpoint_route="/add_face",
    query_params={"face_name": str, "save": bool},
    file_params={"image": ImageFile}
)
srvc_face2face.add_endpoint(
    endpoint_route="/swap_video",
    query_params={"face_name": str, "include_audio": bool, "enhance_face_model": str},
    file_params={"target_video": VideoFile},
    refresh_interval_s=3  # check every 3 seconds for progress
)



from fastsdk.definitions.ai_model import AIModelDescription
from fastsdk.definitions.enums import ModelDomainTag
from fastsdk.web.service_client import ServiceClient
from socaity.settings import DEFAULT_SOCAITY_URL, DEFAULT_REPLICATE_URL
from socaity.api.text.chat.llama3.meta_llama3_schema import MetaCodeLlama3_Input

srvc_codellama_13b = ServiceClient(
    service_urls={
        "socaity": f"{DEFAULT_SOCAITY_URL}/codellama-13b",
        "replicate": f"{DEFAULT_REPLICATE_URL}/meta/codellama-13b",
        "socaity_local": f"http://localhost:8000/v0/codellama-70b-python",
    },
    service_name="codellama-13b",
    model_description=AIModelDescription(
        model_name="codellama-13b",
        model_domain_tags=[ModelDomainTag.TEXT],
    ),
)

srvc_codellama_13b.add_endpoint(endpoint_route="/chat", query_params=MetaCodeLlama3_Input(), refresh_interval_s=5)


srvc_codellama_70b_python = ServiceClient(
    service_urls={
        "socaity": f"{DEFAULT_SOCAITY_URL}/codellama-70b-python",
        "replicate": "https://replicate.com/meta/codellama-70b-python",
    },
    service_name="codellama-70b-python",
    model_description=AIModelDescription(
        model_name="codellama-70b-python",
        model_domain_tags=[ModelDomainTag.TEXT],
    ),
)

srvc_codellama_70b_python.add_endpoint(endpoint_route="/chat", query_params=MetaCodeLlama3_Input(), refresh_interval_s=5)
